@@ -107,21 +107,22 @@
 
 (defun gpt-run-buffer (buffer)
   "Run GPT request with BUFFER content as input.
-Appends streaming output to the buffer."
+Appends streaming output to the buffer.
+Returns the request process when using curl, nil otherwise."
   (with-current-buffer buffer
     ;; Kill any existing process
     (when (and gpt--request-process
                (process-live-p gpt--request-process))
-      (if (y-or-n-p "A GPT process is running here. Kill it and start a new one? ")
+      (if (y-or-n-p "A GPT process is running here.  Kill it and start a new one? ")
           (delete-process gpt--request-process)
-        (user-error "Aborted. Existing GPT process is still running")))
+        (user-error "Aborted.  Existing GPT process is still running")))
     ;; Validate API key
     (gpt-validate-api-key)
     ;; Ensure we have a valid backend (not just non-nil, but correct type)
     (let* ((api-type (gpt--current-api-type))
            (messages (gpt-parse-buffer-messages buffer))
            (options (list :model gpt-model
-                          :max-tokens (string-to-number gpt-max-tokens)
+                          :max-tokens gpt-max-tokens
                           :temperature (string-to-number gpt-temperature)))
            ;; Add provider-specific options
            (options (if (eq api-type 'anthropic)
@@ -129,7 +130,7 @@ Appends streaming output to the buffer."
                          (plist-put
                           (plist-put
                            (plist-put options :thinking-enabled gpt-thinking-enabled)
-                           :thinking-budget (string-to-number gpt-thinking-budget))
+                           :thinking-budget gpt-thinking-budget)
                           :interleaved-thinking gpt-interleaved-thinking)
                          :web-search gpt-web-search)
                       options))
@@ -180,7 +181,9 @@ Appends streaming output to the buffer."
                  (when-let* ((thinking (plist-get content :thinking)))
                    (gpt--insert-stream-output buffer nil thinking))
                  (gpt--insert-stream-output buffer (plist-get content :content) nil))
-               (gpt--finalize-stream buffer t nil)))))))))
+               (gpt--finalize-stream buffer t nil))))))
+      ;; Return the process (nil if using url-retrieve fallback)
+      gpt--request-process)))
 
 (defun gpt-abort-request (&optional buffer)
   "Abort any active request in BUFFER (default current buffer)."
