@@ -132,6 +132,160 @@ If no code fence is found, return TEXT trimmed."
       (setq clean (substring clean 0 (match-beginning 0)))))
     clean))
 
+;;; Search/Replace block editing
+
+(defun gpt--edit-parse-search-replace-blocks (text)
+  "Parse TEXT for SEARCH/REPLACE blocks.
+Returns a list of plists with :search and :replace keys.
+Format expected:
+<<<<<<< SEARCH
+old code
+=======
+new code
+>>>>>>> REPLACE"
+  (let ((blocks '())
+        (pos 0)
+        (pattern (concat "<<<<<<+[ \t]*SEARCH[ \t]*\n"
+                         "\\(\\(?:.*\n\\)*?\\)"
+                         "=======*[ \t]*\n"
+                         "\\(\\(?:.*\n\\)*?\\)"
+                         ">>>>>>>+[ \t]*REPLACE")))
+    (while (string-match pattern text pos)
+      (let ((search-text (match-string 1 text))
+            (replace-text (match-string 2 text)))
+        ;; Remove trailing newline from search (but keep internal structure)
+        (when (string-suffix-p "\n" search-text)
+          (setq search-text (substring search-text 0 -1)))
+        ;; Remove trailing newline from replace
+        (when (string-suffix-p "\n" replace-text)
+          (setq replace-text (substring replace-text 0 -1)))
+        (push (list :search search-text :replace replace-text) blocks))
+      (setq pos (match-end 0)))
+    (nreverse blocks)))
+
+(defun gpt--edit-has-search-replace-blocks (text)
+  "Return non-nil if TEXT has SEARCH/REPLACE blocks."
+  (string-match-p "<<<<<<+[ \t]*SEARCH" text))
+
+(defun gpt--edit-normalize-whitespace (text)
+  "Normalize TEXT for fuzzy matching by collapsing whitespace."
+  (let ((normalized text))
+    ;; Trim each line
+    (setq normalized
+          (mapconcat (lambda (line) (string-trim line))
+                     (split-string normalized "\n")
+                     "\n"))
+    ;; Trim overall
+    (string-trim normalized)))
+
+(defun gpt--edit-find-match (search-text content)
+  "Find SEARCH-TEXT in CONTENT.
+Returns a plist with :start, :end, and :match-type keys.
+:match-type is \\='exact or \\='fuzzy.
+Returns nil if no unique match found."
+  (let ((exact-matches '())
+        (pos 0))
+    ;; Try exact match first
+    (while (setq pos (string-match (regexp-quote search-text) content pos))
+      (push pos exact-matches)
+      (setq pos (1+ pos)))
+    (cond
+     ;; Exactly one exact match
+     ((= (length exact-matches) 1)
+      (list :start (car exact-matches)
+            :end (+ (car exact-matches) (length search-text))
+            :match-type 'exact))
+     ;; Multiple exact matches - ambiguous
+     ((> (length exact-matches) 1)
+      (list :error 'multiple-matches
+            :count (length exact-matches)))
+     ;; No exact match - try fuzzy
+     (t
+      (let* ((fuzzy-matches '())
+             (normalized-search (gpt--edit-normalize-whitespace search-text))
+             (lines (split-string content "\n"))
+             (search-line-count (length (split-string search-text "\n"))))
+        ;; Slide a window over the content lines
+        (cl-loop for i from 0 to (- (length lines) search-line-count)
+                 do (let* ((window (cl-subseq lines i (+ i search-line-count)))
+                           (window-text (mapconcat #'identity window "\n"))
+                           (normalized-window (gpt--edit-normalize-whitespace window-text)))
+                      (when (string= normalized-search normalized-window)
+                        (push (list :line-start i :line-count search-line-count
+                                    :original-text window-text)
+                              fuzzy-matches))))
+        (cond
+         ;; Exactly one fuzzy match
+         ((= (length fuzzy-matches) 1)
+          (let* ((match (car fuzzy-matches))
+                 (line-start (plist-get match :line-start))
+                 (before-lines (cl-subseq lines 0 line-start))
+                 (start-pos (+ (apply #'+ (mapcar #'length before-lines))
+                               line-start)) ; account for newlines
+                 (matched-text (plist-get match :original-text)))
+            (list :start start-pos
+                  :end (+ start-pos (length matched-text))
+                  :match-type 'fuzzy
+                  :original-text matched-text)))
+         ;; Multiple fuzzy matches
+         ((> (length fuzzy-matches) 1)
+          (list :error 'multiple-matches
+                :count (length fuzzy-matches)
+                :match-type 'fuzzy))
+         ;; No match at all
+         (t
+          (list :error 'not-found))))))))
+
+(defun gpt--edit-apply-search-replace (content blocks)
+  "Apply SEARCH/REPLACE BLOCKS to CONTENT.
+Returns a plist with :content (new content) and :errors (list of failed blocks)."
+  (let ((result content)
+        (errors '())
+        (applied 0))
+    (dolist (block blocks)
+      (let* ((search-text (plist-get block :search))
+             (replace-text (plist-get block :replace))
+             (match (gpt--edit-find-match search-text result)))
+        (cond
+         ;; Successful match
+         ((and match (not (plist-get match :error)))
+          (let ((start (plist-get match :start))
+                (end (plist-get match :end)))
+            (setq result (concat (substring result 0 start)
+                                 replace-text
+                                 (substring result end)))
+            (setq applied (1+ applied))))
+         ;; Error
+         (t
+          (push (list :block block :error match) errors)))))
+    (list :content result
+          :errors (nreverse errors)
+          :applied applied)))
+
+(defun gpt--edit-format-apply-errors (errors)
+  "Format ERRORS from search/replace application for display."
+  (mapconcat
+   (lambda (err)
+     (let* ((block (plist-get err :block))
+            (error-info (plist-get err :error))
+            (search-text (plist-get block :search))
+            (preview (if (> (length search-text) 60)
+                         (concat (substring search-text 0 57) "...")
+                       search-text)))
+       (pcase (plist-get error-info :error)
+         ('multiple-matches
+          (format "- Multiple matches (%d) for: %s"
+                  (plist-get error-info :count)
+                  (replace-regexp-in-string "\n" "\\\\n" preview)))
+         ('not-found
+          (format "- Not found: %s"
+                  (replace-regexp-in-string "\n" "\\\\n" preview)))
+         (_
+          (format "- Unknown error for: %s"
+                  (replace-regexp-in-string "\n" "\\\\n" preview))))))
+   errors
+   "\n"))
+
 (defun gpt--edit-generate-diff (original-text new-content)
   "Generate diff between ORIGINAL-TEXT and NEW-CONTENT.
 Returns a plist with :exit-code and :diff-buffer keys.
@@ -182,8 +336,11 @@ Creates an undo boundary so changes can be reverted with \\[undo]."
 
 (defun gpt--edit-handle-choice (choice source-buffer prompt-buffer new-content
                                        original-text base-command history window-config)
-  "Handle user CHOICE (y/n/f) for GPT edit.
-Returns t if the edit should be rerun with feedback."
+  "Handle user CHOICE (y/n/f) for GPT edit on SOURCE-BUFFER.
+PROMPT-BUFFER holds the GPT conversation, NEW-CONTENT is the proposed text.
+ORIGINAL-TEXT is the pre-edit content, BASE-COMMAND the initial instruction.
+HISTORY is prior feedback strings, WINDOW-CONFIG the layout to restore.
+Return t if the edit should be rerun with feedback."
   (pcase choice
     ((or ?y ?Y)
      (when (buffer-live-p prompt-buffer)
@@ -242,10 +399,31 @@ WINDOW-CONFIG is the window configuration to restore after editing."
       (message "GPT edit returned no content.")
       (cl-return-from gpt--finalize-edit))
 
-    ;; Process content and check for changes
-    (let* ((extracted (gpt--edit-extract-code-block raw-output))
-           (new-content (string-trim (gpt--edit-strip-code-fences extracted)))
-           (reran nil))
+    ;; Detect format: search/replace blocks or full file replacement
+    (let* ((has-search-replace (gpt--edit-has-search-replace-blocks raw-output))
+           new-content
+           apply-errors
+           reran)
+
+      (if has-search-replace
+          ;; Search/replace mode
+          (let* ((blocks (gpt--edit-parse-search-replace-blocks raw-output))
+                 (result (gpt--edit-apply-search-replace original-text blocks))
+                 (errors (plist-get result :errors))
+                 (applied (plist-get result :applied)))
+            (setq new-content (plist-get result :content))
+            (setq apply-errors errors)
+            (message "GPT edit: parsed %d search/replace blocks, applied %d."
+                     (length blocks) applied)
+            (when errors
+              (message "GPT edit: %d blocks failed to apply." (length errors))))
+
+        ;; Full file replacement mode
+        (let* ((extracted (gpt--edit-extract-code-block raw-output))
+               (content (string-trim (gpt--edit-strip-code-fences extracted))))
+          (setq new-content content)
+          (message "GPT edit: using full file replacement mode.")))
+
       ;; Preserve trailing newline if original had one
       (when (and (string-suffix-p "\n" original-text)
                  (not (string-suffix-p "\n" new-content)))
@@ -273,6 +451,14 @@ WINDOW-CONFIG is the window configuration to restore after editing."
                   (set-window-configuration window-config))
                 (message "GPT edit: diff produced no changes."))
 
+            ;; Show errors if any blocks failed
+            (when apply-errors
+              (with-current-buffer diff-buffer
+                (goto-char (point-max))
+                (insert "\n\n--- Search/Replace Errors ---\n"
+                        (gpt--edit-format-apply-errors apply-errors)
+                        "\n")))
+
             ;; Display diff and prompt for user action
             (display-buffer diff-buffer)
             (message "GPT edit: review diff and confirm.")
@@ -321,12 +507,22 @@ WINDOW-CONFIG is the window configuration to restore after editing (optional)."
          (prompt (concat
                   "User:\n\n"
                   (format
-                   "You are editing the Emacs buffer \"%s\" (File: %s).\n"
+                   "You are editing the Emacs buffer \"%s\" (File: %s).\n\n"
                    buffer-title file-path)
-                  "Rewrite the entire buffer exactly once so it satisfies the instruction while leaving unrelated content unchanged.\n"
+                  "Choose ONE of these response formats based on the scope of changes:\n\n"
+                  "**For small, targeted changes** (preferred when possible): Use SEARCH/REPLACE blocks:\n"
+                  "```\n"
+                  "<<<<<<< SEARCH\n"
+                  "exact code to find\n"
+                  "=======\n"
+                  "replacement code\n"
+                  ">>>>>>> REPLACE\n"
+                  "```\n"
+                  "You can include multiple SEARCH/REPLACE blocks. The SEARCH text must match exactly (including whitespace).\n\n"
                   (format
-                   "Return only the complete updated buffer enclosed in a ```%s``` fenced code block with no commentary outside the fence.\n\n"
+                   "**For extensive changes**: Return the complete updated buffer in a ```%s``` code block.\n\n"
                    language)
+                  "Important: Do NOT mix formats. Use either SEARCH/REPLACE blocks OR a full file replacement, not both.\n\n"
                   "<instruction>\n"
                   effective-command
                   "\n</instruction>\n"
@@ -343,10 +539,10 @@ WINDOW-CONFIG is the window configuration to restore after editing (optional)."
     (let ((initial-size (with-current-buffer prompt-buffer (point-max))))
       (switch-to-buffer-other-window prompt-buffer)
       (message "GPT edit: generating proposal...")
-      (gpt-run-buffer prompt-buffer)
-      (let ((proc (get-buffer-process prompt-buffer)))
+      ;; gpt-run-buffer returns the process when using curl
+      (let ((proc (gpt-run-buffer prompt-buffer)))
         (unless proc
-          (user-error "Failed to start GPT process"))
+          (user-error "GPT edit requires curl for streaming.  Check that curl is installed and `gpt-use-curl' is t"))
         (let ((original-sentinel (process-sentinel proc))
               (finalized nil))
           (set-process-sentinel
@@ -412,7 +608,7 @@ If there is an active region, append as \"Selected region:\"."
   "Get the output buffer name for a given COMMAND."
   (let* ((truncated-command (substring command 0 (min gpt-buffer-name-length (length command))))
          (ellipsis (if (< (length truncated-command) (length command)) "..." "")))
-    (concat "*gpt"
+    (concat "*" gpt-model
             "[" (number-to-string gpt-buffer-counter) "]: "
             truncated-command
             ellipsis
@@ -456,13 +652,9 @@ CONTEXT-MODE can be one of:
 - \\='current-buffer: Use current buffer as context
 - nil or \\='none: Use no buffer context
 
-By default, uses `gpt-multi-models-default' without prompting.
-With a prefix argument (C-u), prompts to choose models interactively.
-
-Example usage:
-  M-x gpt-chat-multi-models RET current-buffer RET
-  C-u M-x gpt-chat-multi-models    ; Interactive model selection
-  (gpt-chat-multi-models \\='current-buffer)  ; From Lisp
+When PROMPT-FOR-MODELS is non-nil, prompt to choose models interactively;
+otherwise use `gpt-multi-models-default'.  With \\[universal-argument],
+prompts interactively.
 
 The command will create separate output buffers for each model,
 allowing you to compare responses side-by-side."
@@ -490,9 +682,8 @@ allowing you to compare responses side-by-side."
           (unless info
             (user-error "Unknown model: %s" model-name))
           (let* ((model-id (plist-get info :id))
-                 (model-max-tokens (or (plist-get info :max-tokens) "64000"))
-                 (model-thinking-budget (number-to-string (/ (string-to-number model-max-tokens)
-                                                             gpt-thinking-budget-fraction)))
+                 (model-max-tokens (or (plist-get info :max-tokens) 64000))
+                 (model-thinking-budget (/ model-max-tokens gpt-thinking-budget-fraction))
                  ;; Dynamically bind per-run model and settings
                  ;; API type is derived from gpt-model via gpt--current-api-type
                  (gpt-model model-id)
@@ -522,7 +713,7 @@ prompt marker, ready for GPT to generate a response."
 
 ;;;###autoload
 (defun gpt-edit-current-buffer ()
-  "Rewrite the current buffer via GPT and apply changes after review."
+  "Rewrite the current buffer via GPT and show a diff for review."
   (interactive)
   (gpt-validate-api-key)
   (let* ((raw-command (gpt-read-command 'current-buffer t))
@@ -633,6 +824,8 @@ CONTEXT-MODE can be:
   "Toggle extended thinking mode for Anthropic models (enabled by default)."
   (interactive)
   (setq gpt-thinking-enabled (not gpt-thinking-enabled))
+  (when (eq (gpt--current-api-type) 'anthropic)
+    (gpt-update-backend))
   (message "Extended thinking mode %s"
            (if gpt-thinking-enabled "enabled" "disabled")))
 
@@ -641,6 +834,8 @@ CONTEXT-MODE can be:
   "Toggle interleaved thinking mode for Anthropic models (enabled by default)."
   (interactive)
   (setq gpt-interleaved-thinking (not gpt-interleaved-thinking))
+  (when (eq (gpt--current-api-type) 'anthropic)
+    (gpt-update-backend))
   (message "Interleaved thinking mode %s"
            (if gpt-interleaved-thinking "enabled" "disabled")))
 
@@ -649,6 +844,8 @@ CONTEXT-MODE can be:
   "Toggle web search for Anthropic models."
   (interactive)
   (setq gpt-web-search (not gpt-web-search))
+  (when (eq (gpt--current-api-type) 'anthropic)
+    (gpt-update-backend))
   (message "Web search %s"
            (if gpt-web-search "enabled" "disabled")))
 
