@@ -3,8 +3,8 @@
 ;; Copyright (C) 2022-2025 Andreas Stuhlmueller
 
 ;; Author: Andreas Stuhlmueller <emacs@stuhlmueller.org>
-;; Package-Version: 20260111.423
-;; Package-Revision: fb5b7a833116
+;; Package-Version: 20260207.633
+;; Package-Revision: 9d4752674e6c
 ;; Keywords: openai, anthropic, claude, language, copilot, convenience, tools
 ;; URL: https://github.com/stuhlmueller/gpt.el
 ;; License: MIT
@@ -45,6 +45,9 @@
 
 (require 'gpt-core)
 (require 'gpt-backend)
+(require 'gpt-openai)
+(require 'gpt-anthropic)
+(require 'gpt-google)
 (require 'gpt-http)
 (require 'gpt-api)
 (require 'gpt-ui)
